@@ -1,6 +1,6 @@
 #ifndef GCC_TM_H
 #define GCC_TM_H
-#define TARGET_CPU_DEFAULT (TARGET_CPU_generic | ((  AARCH64_FL_FOR_ARCH8) << 6))
+#define TARGET_CPU_DEFAULT (TARGET_CPU_generic | ((  AARCH64_FL_FOR_ARCH8) << TARGET_CPU_NBITS))
 #ifndef LIBC_GLIBC
 # define LIBC_GLIBC 1
 #endif
